#pragma once

#include "Common.h"
#include "Points.h"

enum METHOD_ID
{
	SBQ, DBQ, QE
};

// hamming distance function
__forceinline int compute_HD(bitset<BCODE_LEN> &a, bitset<BCODE_LEN> &b)
{
	return (a ^ b).count();
}

// quadra embedding distance function
__forceinline int compute_QED(bitset<BCODE_LEN> &a, bitset<BCODE_LEN> &b)
{
	bitset<BCODE_LEN> tmp1, tmp2, tmp;
	tmp = (a ^ b) << (BCODE_LEN / 2);
	tmp1 = tmp & (a & b); // # of pairs with distance 2
	tmp2 = tmp & (a ^ b); // # of pairs with distance 1
	return 2 * tmp1.count() + tmp2.count();
}

class BinaryHash
{
public:
	int dim;				// dimensionality of original points
	int p_num;				// number of projections
	REAL_TYPE **p_matrix;	// projection matrix
	METHOD_ID method_id;	// method id: Single Bit Quantization (SBQ), Double Bit Quantization (DBQ), Quadra-Embedding (QE)

	// thresholds for DBQ and QE
	REAL_TYPE *threshold1;	
	REAL_TYPE *threshold2;
	REAL_TYPE *threshold3;

	BinaryHash(int _dim, METHOD_ID _method_id);
	~BinaryHash();
	void initialize_LSH();
	void initialize_matrix(char *pM_path);
	void compute_BCode(REAL_TYPE *x, bitset<BCODE_LEN> &y);
	void train_QE(int num_train_samples, Points &tps);
	void train_DBQ(int num_train_samples, Points &tps);
};
