#pragma once

#include "Common.h"

class Points
{
public :
	int nP;
	int dim;
	REAL_TYPE **d;
	
	void initialize(int _nP, int _dim)
	{
		nP = _nP;		
		dim = _dim;
		d = new REAL_TYPE *[nP];
		for (int i = 0; i < nP; i++)
		{
			d[i] = new REAL_TYPE[dim];
		}
	}

	// this function is for read point set from file
	// format:
	// (num. points) (dimensionality)
	// v0 (floats, number of elements is equal to dimensionality)
	// v1
	// ...
	void initialize_from_file(char *fileName)
	{
		FILE *input = fopen( fileName , "rb" );
		fread( &nP , sizeof(int) , 1 , input );
		fread( &dim , sizeof(int) , 1 , input );
		d = new REAL_TYPE * [ nP ];
		for(int i=0;i<nP;i++)
		{
			d[i] = new REAL_TYPE [ dim ];
		}
		float *tmp = new float [ dim ];
		for(int i=0;i<nP;i++)
		{
			fread( tmp , sizeof(float) , dim , input );
			for(int k=0;k<dim;k++)
			{
				d[i][k] = (REAL_TYPE)( tmp[k] );
			}
		}
		fclose(input);
		delete [] tmp;
	}

	void sample_points(int num_sample_points, Points *tps) 
	{
		tps->initialize(num_sample_points, dim);	
		
		bool *checkList = new bool[nP];
		SetVector_Val<bool>(checkList, nP, true);
		int index;

		// sampling training set
		for (int i = 0; i < num_sample_points; i++)
		{
			while(true)
			{
				index = Rand_Uniform_Int(0, nP - 1);
				if(checkList[index])
				{
					checkList[index] = false;
					break;
				}
			}
			SetVector_Vec<REAL_TYPE>(tps->d[i], d[index], dim);		
		}
		delete[] checkList;
	}

	// computing center of points for zero centering
	void compute_center(REAL_TYPE *center)
	{
		double *tCenter = new double [dim];
		SetVector_Val<double>( tCenter , dim , 0.0 );
		for(int i=0;i<nP;i++)
		{
			for(int k=0;k<dim;k++)
			{
				tCenter[k] += d[i][k];
			}
		}
		for(int k=0;k<dim;k++)
		{
			tCenter[k] /= (double)(nP);
			center[k] = (REAL_TYPE)( tCenter[k] );
		}
		delete [] tCenter;
	}

	void release_memory()
	{
		for(int i=0;i<nP;i++)
		{
			delete [] d[i];
		}
		delete [] d;
	}

	
};